@@ -1,6 +1,6 @@
 /*
  * QCPUFreq - a simple cpufreq GUI
- * Copyright (C) 2010 Daniel Klaffenbach <daniel.klaffenbach@cs.tu-chemnitz.de>
+ * Copyright (C) 2010 Daniel Klaffenbach <danielklaffenbach@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,13 +28,12 @@
 #endif
 
 #define APPNAME "QCPUFreq"
-#define APPVERSION "0.3.4"
+#define APPVERSION "0.4.0"
+
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
     ui(new Ui::MainWindow),
-    //do not allow overclocking per default
-    allowOverclocking(false),
     //create helper process
     helperProcess( this ),
     //create a new, stackable help window
@@ -42,9 +41,7 @@ MainWindow::MainWindow(QWidget *parent) :
     //create UI refresh timer
     refreshTimer( this ),
     //create a QGraphicsScene for the little chip icon
-    scene( this ),
-    //show errors about the sudo setup only once
-    showSudoError( true )
+    scene( this )
 {
     //this is a stacked window on Maemo 5
     #if defined(Q_WS_MAEMO_5)
@@ -53,11 +50,14 @@ MainWindow::MainWindow(QWidget *parent) :
 
     ui->setupUi(this);
 
-    init();
-    refresh();
+    //Settings widget
+    settings = new Settings;
+    settings->hide();
 
-    // enable auto rotation
-    setAutoRotation();
+    init();
+
+    //applies the settings from the settings dialog
+    applySettings();
 
     //initialize orientation
     orientationChanged();
@@ -71,6 +71,9 @@ MainWindow::MainWindow(QWidget *parent) :
     #endif
     helpWindow.setWindowFlags( windowFlags() | Qt::Window );
 
+    //show errors about the sudo setup only once
+    showSudoError = true;
+
     //connect signals and slots
     connect(ui->actionHelp, SIGNAL(triggered()), this, SLOT(showHelp()));
     connect( ui->actionAbout, SIGNAL(triggered()), this, SLOT(about()) );
@@ -78,18 +81,15 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
     connect(ui->sr_box, SIGNAL(clicked()), this, SLOT(setSmartReflex()));
     connect(&refreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
-    connect(ui->actionOverclocking, SIGNAL(toggled(bool)), this, SLOT(setOverclocking()));
-
-
-    //disable overclocking button on vanilla kernels
-    if (!powerKernel) {
-        ui->actionOverclocking->setDisabled(true);
-    }
+    connect(ui->actionSave, SIGNAL(triggered()), this, SLOT(save()));
+    connect(ui->actionSettings, SIGNAL(triggered()), this, SLOT(showSettings()));
+    connect(settings, SIGNAL(settingsChanged()), this, SLOT(applySettings()));
 
 }
 
 MainWindow::~MainWindow()
 {
+    delete settings;
     delete ui;
 }
 
@@ -126,19 +126,38 @@ void MainWindow::adjustFreq()
 
     //check for overclocking
     #if defined(Q_WS_MAEMO_5)
-    if (this->allowOverclocking == false && newmax > 600000) {
-        QMaemo5InformationBox::information(this, tr( "You need to enable overclocking in QCPUFreq's menu for setting frequencies above 600MHz!"), 0);
+    if (!settings->useOverclocking() && newmax > 600000) {
+        QMaemo5InformationBox::information(this, tr( "You need to enable overclocking in QCPUFreq's settings in order to set frequencies above 600MHz!"), 0);
         refresh();
         return;
     }
     #endif
 
+    //check for 599MHz <-> 600MHz problem on power kernels
+    if (max == "600000" && settings->usePowerKernel()) {
+        //we really need to set the maximum to 599MHz
+        max = "599000";
+    }
+
     callHelper( "set_maxfreq", max );
 
     refresh();
 }
 
 
+/**
+  * SLOT: applies the settings from the Settings dialog.
+  */
+void MainWindow::applySettings()
+{
+    setAutoRotation();
+    setAdvancedTemperature();
+
+    //refresh the GUI after applying the settings
+    refresh();
+}
+
+
 /**
   * Calls the QCPUFreq helper script with "sudo action param"
   *
@@ -305,7 +324,6 @@ int MainWindow::getSmartReflexState()
   *  - scalingSteps
   *  - scalingFrequencies
   *  - minFreq
-  *  - powerKernel
   */
 void MainWindow::init()
 {
@@ -339,11 +357,9 @@ void MainWindow::init()
         this->minFreq = min.toInt();
     }
 
-    //check if we are using a power kernel
-    if ( getScalingFreq(getScalingSteps()) > 600000 ) {
-        this->powerKernel = true;
-    } else {
-        this->powerKernel = false;
+    //enable save option on power kernels
+    if (settings->usePowerKernel()) {
+        ui->actionSave->setEnabled(true);
     }
 }
 
@@ -410,8 +426,6 @@ void MainWindow::refresh()
     ui->freq_adjust->setMinimum( 1 );
     ui->freq_adjust->setMaximum( getScalingSteps() );
     ui->freq_adjust->setSliderPosition( getScalingStep(getMaxFreq()) );
-
-    //ui->retranslateUi(this);
 }
 
 
@@ -445,32 +459,41 @@ void MainWindow::orientationChanged()
 
 
 /**
-  * Enables the auto-rotation feature of Maemo5 devices
+  * Saves the current maximim frequency as default (only on power kernel).
   */
-void MainWindow::setAutoRotation()
+void MainWindow::save()
 {
-#if defined(Q_WS_MAEMO_5)
-    setAttribute(Qt::WA_Maemo5AutoOrientation, true);
-#endif
+    if (settings->usePowerKernel()) {
+        callHelper( "save", "null" );
+        #if defined(Q_WS_MAEMO_5)
+            QMaemo5InformationBox::information(this, tr( "The current frequency settings have been saved as default." ), QMaemo5InformationBox::DefaultTimeout);
+        #endif
+    }
 }
 
 
 /**
-  * SLOT: enable/disable overclocking.
+  * Checks the settings if the "bq27x00_battery" needs to be loaded.
   */
-void MainWindow::setOverclocking()
+void MainWindow::setAdvancedTemperature()
 {
-    if (ui->actionOverclocking->isChecked()) {
-        #if defined(Q_WS_MAEMO_5)
-        QMaemo5InformationBox::information(this, tr( "Please note that overclocking voids your warranty and may break your device! Be careful!"), 0);
-        #endif
-        this->allowOverclocking = true;
-    } else {
-        this->allowOverclocking = false;
+    if (settings->usePowerKernel() && settings->useAdvancedTemperature()) {
+       callHelper( "load_bq27", "null" );
     }
 }
 
 
+/**
+  * Enables or disables the auto-rotation feature of Maemo5 devices.
+  */
+void MainWindow::setAutoRotation()
+{
+#if defined(Q_WS_MAEMO_5)
+    setAttribute(Qt::WA_Maemo5AutoOrientation, settings->useAutoRotate());
+#endif
+}
+
+
 /**
   * SLOT: Enables or disables Smart Reflex(tm) after pressing sr_btn
   */
@@ -500,6 +523,16 @@ void MainWindow::showHelp()
 }
 
 
+/**
+  * SLOT: displays the settings widget
+  */
+void MainWindow::showSettings()
+{
+    settings->reset();
+    settings->show();
+}
+
+
 /**
   * Returns true when the device is in portrait mode
   */
